@@ -7,7 +7,7 @@
 //!
 //! ```toml
 //! [dependencies]
-//! static_assertions = "0.2.2"
+//! static_assertions = "0.2.3"
 //! ```
 //!
 //! and this to your crate root:
@@ -167,6 +167,7 @@
 //!
 //! - [`assert_eq_size`]
 //! - [`assert_obj_safe`]
+//! - [`assert_impl`]
 //! - [`const_assert`]
 //! - [`const_assert_eq`]
 //!
@@ -223,16 +224,16 @@ pub extern crate core as _core;
 /// ```
 #[macro_export]
 macro_rules! assert_eq_size {
-    ($x:ty, $($xs:ty),+) => {
-        #[allow(unused_unsafe)]
+    ($x:ty, $($xs:ty),+ $(,)*) => {
+        #[allow(unknown_lints, forget_copy, unused_unsafe, useless_transmute)]
         unsafe {
             use $crate::_core::mem::{forget, transmute, uninitialized};
             $(forget::<$xs>(transmute(uninitialized::<$x>()));)+
         }
     };
-    ($label:ident; $x:ty, $($xs:ty),+) => {
+    ($label:ident; $($xs:tt)+) => {
         #[allow(dead_code, non_snake_case)]
-        fn $label() { assert_eq_size!($x, $($xs),+) }
+        fn $label() { assert_eq_size!($($xs)+); }
     };
 }
 
@@ -253,8 +254,8 @@ macro_rules! assert_eq_size {
 /// ```
 #[macro_export]
 macro_rules! assert_eq_size_ptr {
-    ($x:expr, $($xs:expr),+) => {
-        #[allow(unused_unsafe)]
+    ($x:expr, $($xs:expr),+ $(,)*) => {
+        #[allow(unknown_lints, forget_copy, unused_unsafe, useless_transmute)]
         unsafe {
             use $crate::_core::{mem, ptr};
             let mut copy = ptr::read($x);
@@ -286,7 +287,7 @@ macro_rules! assert_eq_size_ptr {
 /// ```
 #[macro_export]
 macro_rules! assert_eq_size_val {
-    ($x:expr, $($xs:expr),+) => {
+    ($x:expr, $($xs:expr),+ $(,)*) => {
         assert_eq_size_ptr!(&$x, $(&$xs),+);
     }
 }
@@ -310,22 +311,23 @@ macro_rules! assert_eq_size_val {
 /// ```
 #[macro_export]
 macro_rules! const_assert {
-    ($($xs:expr),+) => {
+    ($($xs:expr),+ $(,)*) => {
+        #[allow(unknown_lints, eq_op)]
         let _ = [(); 0 - (!($($xs)&&+) as usize)];
     };
-    ($label:ident; $($xs:expr),+) => {
+    ($label:ident; $($xs:tt)+) => {
         #[allow(dead_code, non_snake_case)]
-        fn $label() { const_assert!($($xs),+); }
+        fn $label() { const_assert!($($xs)+); }
     };
 }
 
 /// Asserts at compile-time that the constants are equal in value.
 #[macro_export]
 macro_rules! const_assert_eq {
-    ($x:expr, $($xs:expr),+) => {
+    ($x:expr, $($xs:expr),+ $(,)*) => {
         const_assert!($($x == $xs),+);
     };
-    ($label:ident; $x:expr, $($xs:expr),+) => {
+    ($label:ident; $x:expr, $($xs:expr),+ $(,)*) => {
         const_assert!($label; $($x == $xs),+);
     };
 }
@@ -336,12 +338,12 @@ macro_rules! const_assert_eq {
 /// prevent it from being used as an object.
 #[macro_export]
 macro_rules! assert_obj_safe {
-    ($($xs:ty),+) => {
+    ($($xs:ty),+ $(,)*) => {
         $(let _: Option<&$xs> = None;)+
     };
-    ($label:ident; $($xs:ty),+) => {
+    ($label:ident; $($xs:tt)+) => {
         #[allow(dead_code, non_snake_case)]
-        fn $label() { assert_obj_safe!($($xs),+); }
+        fn $label() { assert_obj_safe!($($xs)+); }
     };
 }
 
@@ -353,7 +355,7 @@ macro_rules! assert_impl {
         $(
             $y:ident
             $(< $($args:ty),+ $(,)* >)*
-        ),+
+        ),+ $(,)*
     ) => {
         $({
             fn assert_impl<T: ?Sized + $y $(< $($args),+ >)* >() {}
